/*
 * CrudContainer
 * Copyright (C) 2011 by Apaq
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package dk.apaq.filter.util;

import dk.apaq.filter.Filter;
import dk.apaq.filter.FiltrationItem;
import dk.apaq.filter.core.SimpleFiltrationItem;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Pattern;
/**
 *
 */
public class FilterUtil {

    private static final Pattern BEANPATTERN = Pattern.compile("get[A-Z][a-zA-Z0-9]*");

    public static FiltrationItem buildItemFromBean(Object object) {
        SimpleFiltrationItem item = new SimpleFiltrationItem();

        Method[] methods = object.getClass().getMethods();
        for(Method method : methods) {
            if(method.getParameterTypes().length ==0 &&
                    method.getReturnType() != Void.class &&
                    BEANPATTERN.matcher(method.getName()).matches()) {
                try {
                    Object returnVal = method.invoke(object);
                    String propertyName = method.getName().substring(3, 4).toLowerCase() + method.getName().substring(4);
                    item.setProperty(propertyName, returnVal);
                } catch (Exception ex) {
                    Logger.getLogger(FilterUtil.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
        return item;
    }
    
    public static <T> List<T> filterCollection(Collection<T> c, Filter filter) {
        List<T> result = new ArrayList<T>();
        for(T element : c) {
            FiltrationItem fi = FilterUtil.buildItemFromBean(element);
            if(filter.passesFilter(fi)) {
                result.add(element);
            }
        }
        return result;
    }
}
